package aps_engine

import (
	"fmt"
	"leit.com/aps_engine/service"
	"testing"
	"time"
)

func TestScheduler(t *testing.T){
	var se service.SchedulerSrv

	// 初始化调度环境
	LoadTestEnvData(&se)
	se.SchedRule.ToolConstraintToggle = true

	// 启动订单调度引擎
	if err := se.Run(); err != nil{
		fmt.Println("调度错误:", err)
	}
	// 打印计划结果e
	fmt.Println("=========================================================>")
	for _, order := range se.OrderDict {
		order.Print()
	}
	for _, wp := range se.WorkPlaceDict{
		wp.Print()
	}
	for _, t := range se.ToolDict{
		t.Print()
	}
}

// 初始化测试环境，创建一些测试数据
func LoadTestEnvData(se *service.SchedulerSrv)(err error){
	var(
		wp		*service.WorkPlaceSrv
		wpg 	*service.WorkPlaceGrpSrv
		t		*service.ToolSrv
		tg		*service.ToolGrpSrv
		art		*service.ArticleSrv
		order	*service.OrderSrv
		prevop	*service.OperationSrv
		op		*service.OperationSrv
		opr 	service.OperationRelSrv
		st,et 	time.Time
		tls		service.TimeLineSrv
	)
	// 初始化工厂号及计划区间
	st = time.Now()
	et = time.Now().Add(time.Hour*24*10)
	se.Init(100, st, et)

	// 创建工位组
	wpg	= &service.WorkPlaceGrpSrv{}
	wpg.Init(10)

	// 创建工位
	wp = &service.WorkPlaceSrv{}
	wp.Init(1,10)

	st = time.Now()
	et = time.Now().Add(time.Hour*24*10)
	wp.TimeCurve.Init(st,et)

	tls = service.TimeLineSrv{}
	st = time.Now()
	et = time.Now().Add(time.Hour*24)
	tls.Init(st, et)
	wp.TimeCurve.TimeLineArray = append(wp.TimeCurve.TimeLineArray, tls)

	tls = service.TimeLineSrv{}
	st = time.Now().Add(time.Hour*28)
	et = time.Now().Add(time.Hour*48)
	tls.Init(st, et)

	wp.TimeCurve.TimeLineArray = append(wp.TimeCurve.TimeLineArray, tls)
	se.WorkPlaceDict[wp.WorkPlaceNr] = wp
	wpg.WorkPlaceDict[wp.WorkPlaceNr] = wp

	wp = &service.WorkPlaceSrv{}
	wp.Init(2,10)

	st = time.Now()
	et = time.Now().Add(time.Hour*24*10)
	wp.TimeCurve.Init(st,et)

	tls = service.TimeLineSrv{}
	st = time.Now()
	et = time.Now().Add(time.Hour*16)
	tls.Init(st, et)
	wp.TimeCurve.TimeLineArray = append(wp.TimeCurve.TimeLineArray, tls)

	tls = service.TimeLineSrv{}
	st = time.Now().Add(time.Hour*24)
	et = time.Now().Add(time.Hour*46)
	tls.Init(st, et)

	wp.TimeCurve.TimeLineArray = append(wp.TimeCurve.TimeLineArray, tls)
	se.WorkPlaceDict[wp.WorkPlaceNr] = wp
	wpg.WorkPlaceDict[wp.WorkPlaceNr] = wp
	se.WorkPlaceGrpDict[wpg.WorkPlaceGrpNr] = wpg

	wpg	= &service.WorkPlaceGrpSrv{}
	wpg.Init(20)

	wp = &service.WorkPlaceSrv{}
	wp.Init(3,20)

	st = time.Now()
	et = time.Now().Add(time.Hour*24*10)
	wp.TimeCurve.Init(st,et)

	tls = service.TimeLineSrv{}
	st = time.Now()
	et = time.Now().Add(time.Hour*20)
	tls.Init(st, et)
	wp.TimeCurve.TimeLineArray = append(wp.TimeCurve.TimeLineArray, tls)

	tls = service.TimeLineSrv{}
	st = time.Now().Add(time.Hour*22)
	et = time.Now().Add(time.Hour*44)
	tls.Init(st, et)

	wp.TimeCurve.TimeLineArray = append(wp.TimeCurve.TimeLineArray, tls)
	se.WorkPlaceDict[wp.WorkPlaceNr] = wp
	wpg.WorkPlaceDict[wp.WorkPlaceNr] = wp
	se.WorkPlaceGrpDict[wpg.WorkPlaceGrpNr] = wpg

	wpg	= &service.WorkPlaceGrpSrv{}
	wpg.Init(30)

	wp = &service.WorkPlaceSrv{}
	wp.Init(4,30)

	st = time.Now()
	et = time.Now().Add(time.Hour*24*15)
	wp.TimeCurve.Init(st,et)

	tls = service.TimeLineSrv{}
	st = time.Now()
	et = time.Now().Add(time.Hour*21)
	tls.Init(st, et)
	wp.TimeCurve.TimeLineArray = append(wp.TimeCurve.TimeLineArray, tls)

	tls = service.TimeLineSrv{}
	st = time.Now().Add(time.Hour*23)
	et = time.Now().Add(time.Hour*50)
	tls.Init(st, et)

	wp.TimeCurve.TimeLineArray = append(wp.TimeCurve.TimeLineArray, tls)
	se.WorkPlaceDict[wp.WorkPlaceNr] = wp
	wpg.WorkPlaceDict[wp.WorkPlaceNr] = wp

	wp = &service.WorkPlaceSrv{}
	wp.Init(5,30)

	st = time.Now()
	et = time.Now().Add(time.Hour*24*10)
	wp.TimeCurve.Init(st,et)

	tls = service.TimeLineSrv{}
	st = time.Now()
	et = time.Now().Add(time.Hour*17)
	tls.Init(st, et)
	wp.TimeCurve.TimeLineArray = append(wp.TimeCurve.TimeLineArray, tls)

	tls = service.TimeLineSrv{}
	st = time.Now().Add(time.Hour*25)
	et = time.Now().Add(time.Hour*52)
	tls.Init(st, et)

	wp.TimeCurve.TimeLineArray = append(wp.TimeCurve.TimeLineArray, tls)
	se.WorkPlaceDict[wp.WorkPlaceNr] = wp
	wpg.WorkPlaceDict[wp.WorkPlaceNr] = wp
	se.WorkPlaceGrpDict[wpg.WorkPlaceGrpNr] = wpg

	// 创建工具
	t = &service.ToolSrv{}
	t.Init(1, 10)

	tls = service.TimeLineSrv{}
	st = time.Now().Add(time.Hour*5)
	et = time.Now().Add(time.Hour*10)
	t.UnAvailTimeLineArray = append(t.UnAvailTimeLineArray, tls)
	se.ToolDict[t.ToolNr] = t

	// 创建工具组
	tg = &service.ToolGrpSrv{}
	tg.Init(10)
	tg.ToolIdxDict[t.ToolNr] = t.ToolNr
	se.ToolGrpDict[tg.ToolGrpNr] = tg

	// 创建工具
	t = &service.ToolSrv{}
	t.Init(2, 20)

	tls = service.TimeLineSrv{}
	st = time.Now().Add(time.Hour*6)
	et = time.Now().Add(time.Hour*12)
	t.UnAvailTimeLineArray = append(t.UnAvailTimeLineArray, tls)
	se.ToolDict[t.ToolNr] = t

	// 创建工具组
	tg = &service.ToolGrpSrv{}
	tg.Init(20)
	tg.ToolIdxDict[t.ToolNr] = t.ToolNr
	se.ToolGrpDict[tg.ToolGrpNr] = tg

	// 创建物料
	art = &service.ArticleSrv{}
	art.Init("art001",100)
	se.ArticleDict[art.ArtId] = art

	// 创建工单及工序
	order = &service.OrderSrv{OrderId: "100001", ProjectId: "G18"}
	order.OperationArray 	= []service.OperationSrv{}
	order.OperationDict		= make(map[string]*service.OperationSrv)

	op = &service.OperationSrv{OrderId: "100001",OperationNr: 10, SplitNr: 0,PlanQty: 10, WorkPlaceGrpNr: 10, ToolGrpNr: 10, PlanDuration: time.Hour * 4}
	op.PrevOpDict 			= make(map[string]service.OperationRelSrv)
	op.NextOpDict			= make(map[string]service.OperationRelSrv)
	order.OperationArray 	= append(order.OperationArray, *op)
	order.OperationDict[op.GetKey()] = op

	prevop = op

	opr = service.OperationRelSrv{OrderId: "100001", OperationNr: 20, SplitNr: 0, Relation: service.OP_LINK_END_START}
	prevop.NextOpDict[opr.GetFullKey()] = opr

	op = &service.OperationSrv{OrderId: "100001",OperationNr: 20, SplitNr: 0,PlanQty: 10, WorkPlaceGrpNr: 20, PlanDuration: time.Hour * 6}
	op.PrevOpDict 			= make(map[string]service.OperationRelSrv)
	opr = service.OperationRelSrv{OrderId: "100001", OperationNr: 10, SplitNr: 0, Relation: service.OP_LINK_END_START}
	op.PrevOpDict[opr.GetFullKey()] = opr

	op.NextOpDict			= make(map[string]service.OperationRelSrv)
	order.OperationArray 	= append(order.OperationArray, *op)
	order.OperationDict[op.GetKey()] = op

	prevop = op

	opr = service.OperationRelSrv{OrderId: "100001", OperationNr: 30, SplitNr: 0, Relation: service.OP_LINK_END_START}
	prevop.NextOpDict[opr.GetFullKey()] = opr

	op = &service.OperationSrv{OrderId: "100001",OperationNr: 30, SplitNr: 0,PlanQty: 10, WorkPlaceGrpNr: 30, PlanDuration: time.Hour * 8}
	op.PrevOpDict 			= make(map[string]service.OperationRelSrv)
	opr = service.OperationRelSrv{OrderId: "100001", OperationNr: 20, SplitNr: 0, Relation: service.OP_LINK_END_START}
	op.PrevOpDict[opr.GetFullKey()] = opr

	op.NextOpDict			= make(map[string]service.OperationRelSrv)
	order.OperationArray 	= append(order.OperationArray, *op)
	order.OperationDict[op.GetKey()] = op

	se.OrderDict[order.OrderId] = order

	order = &service.OrderSrv{OrderId: "100002", ProjectId: "G18"}
	order.OperationArray 	= []service.OperationSrv{}
	order.OperationDict		= make(map[string]*service.OperationSrv)

	op = &service.OperationSrv{OrderId: "100002",OperationNr: 10, SplitNr: 0,PlanQty: 10, WorkPlaceGrpNr: 10, ToolGrpNr: 10, PlanDuration: time.Hour * 5}
	op.PrevOpDict 			= make(map[string]service.OperationRelSrv)
	op.NextOpDict			= make(map[string]service.OperationRelSrv)
	order.OperationArray 	= append(order.OperationArray, *op)
	order.OperationDict[op.GetKey()] = op

	prevop = op

	opr = service.OperationRelSrv{OrderId: "100002", OperationNr: 20, SplitNr: 0, Relation: service.OP_LINK_END_START}
	prevop.NextOpDict[opr.GetFullKey()] = opr

	op = &service.OperationSrv{OrderId: "100002",OperationNr: 20, SplitNr: 0,PlanQty: 10, WorkPlaceGrpNr: 20, PlanDuration: time.Hour * 7}
	op.PrevOpDict 			= make(map[string]service.OperationRelSrv)
	opr = service.OperationRelSrv{OrderId: "100002", OperationNr: 10, SplitNr: 0, Relation: service.OP_LINK_END_START}
	op.PrevOpDict[opr.GetFullKey()] = opr

	op.NextOpDict			= make(map[string]service.OperationRelSrv)
	order.OperationArray 	= append(order.OperationArray, *op)
	order.OperationDict[op.GetKey()] = op

	prevop = op

	opr = service.OperationRelSrv{OrderId: "100002", OperationNr: 30, SplitNr: 0, Relation: service.OP_LINK_END_START}
	prevop.NextOpDict[opr.GetFullKey()] = opr

	op = &service.OperationSrv{OrderId: "100002",OperationNr: 30, SplitNr: 0,PlanQty: 10, WorkPlaceGrpNr: 30, PlanDuration: time.Hour * 9}
	op.PrevOpDict 			= make(map[string]service.OperationRelSrv)
	opr = service.OperationRelSrv{OrderId: "100002", OperationNr: 20, SplitNr: 0, Relation: service.OP_LINK_END_START}
	op.PrevOpDict[opr.GetFullKey()] = opr

	op.NextOpDict			= make(map[string]service.OperationRelSrv)
	order.OperationArray 	= append(order.OperationArray, *op)
	order.OperationDict[op.GetKey()] = op
	se.OrderDict[order.OrderId] = order

	// 初始化调度队列
	se.SchedEventQueue = service.NewSchedQueue()

	// 遍历设备时间曲线，生成设备状态事件，推入队列
	for _, wp_p := range se.WorkPlaceDict{
		for _, tls := range wp_p.TimeCurve.TimeLineArray{
			evt1 := wp_p.GetAvailEvent(tls.StartTime)
			se.SchedEventQueue.Insert(evt1)
			evt2 := wp_p.GetUnAvailEvent(tls.EndTime)
			se.SchedEventQueue.Insert(evt2)
		}
	}

	// 载入订单首工序事件
	for _, ord_p := range se.OrderDict{
		for _, op_p := range ord_p.OperationDict {
			if len(op_p.PrevOpDict) <= 0 {
				evt := op_p.GetPlanStartEvent(time.Now())
				opet := evt.(service.OperationEvent)
				fmt.Println("压入订单工序事件：",opet.OrderId,opet.OperationNr, opet.TriggerTime)
				se.SchedEventQueue.Insert(evt)
			}
		}
	}

	return
}

